/*
 * Shader.h
 *
 * Copyright (C) 2009 Matthias Schiffer <matthias@gamezock.de>
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along
 * with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef ZOOM_SHADER_H_
#define ZOOM_SHADER_H_

#include "gl.h"

#include <string>
#include <boost/shared_ptr.hpp>

namespace Zoom {

class Shader {
  public:
    static boost::shared_ptr<Shader> load(const std::string &vertexShader, const std::string &fragmentShader);

    static void enable(boost::shared_ptr<Shader> shader) {
      if(shader)
        glUseProgramObjectARB(shader->handle);
      else
        glUseProgramObjectARB(0);
    }

    static void disable() {
      enable(boost::shared_ptr<Shader>());
    }

    ~Shader() {
      glDeleteObjectARB(handle);
    }

  private:
    static GLhandleARB loadShader(const std::string &name, GLenum type);
    static void printInfoLog(GLhandleARB obj);

    Shader(GLhandleARB handle0) : handle(handle0) {}

    // Prevent shallow copy
    Shader(const Shader &o);
    Shader& operator=(const Shader &o);

    GLhandleARB handle;
};

}

#endif /* ZOOM_SHADER_H_ */
