/*
 * Texture.cpp
 *
 * Copyright (C) 2009 Matthias Schiffer <matthias@gamezock.de>
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along
 * with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "Texture.h"
#include "gl.h"
#include <GL/glpng.h>

namespace Zoom {

std::map<std::string, unsigned> Texture::textures;

unsigned Texture::loadTexture(const std::string &name) {
  std::map<std::string, unsigned>::iterator it = textures.find(name);

  if(it != textures.end())
    return it->second;

  pngInfo info;
  unsigned texture = pngBind(("tex/" + name).c_str(), PNG_NOMIPMAP, PNG_ALPHA, &info, GL_REPEAT, GL_LINEAR, GL_LINEAR);

  if(texture) {
    textures.insert(std::make_pair(name, texture));
  }

  return texture;
}

}
